#include "/lib/core.glsl"
#include "/lib/config.glsl"

// todo!() Just disable this program entirely when DCG is disabled
layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
const ivec3 workGroups = ivec3(1, 1, 1);

#if DCG_EXP || DCG_COLOR || DCG_BP
	// uniform int dcgLod;
	uniform float frameTime;
	uniform sampler2D tempHDRS;

	restrict layout(shared, binding = 0) buffer dcgBuffer {
		vec3 max_color;
		float max_luma, min_val;
	} dcg;

	#include "/lib/luma.glsl"
	#include "/lib/tonemap.glsl"
#endif

void main() {
	#if DCG_EXP || DCG_COLOR || DCG_BP
		float min_val = 1.0;
		vec3 max_color = vec3(0.0);

		for(uint x = 0u; x < 64u; ++x) {
			for(uint y = 0u; y < 64u; ++y) {
				vec3 color = textureLod(tempHDRS, vec2(x, y) / 64.0, 0.0).rgb;

				min_val = min(min(min_val, color.r), min(color.g, color.b));
				max_color = mix(max_color, max(max_color, color), DCG_PEAK_SENS * 0.00025);
			}
		}

		min_val = min(tonemap((min_val * 0.1 * DCG_BP).rrr).r, DCG_MAX_BP);
		max_color = clamp(pow(max_color, vec3(0.06 * DCG_EXP)), DCG_MIN_PEAK, DCG_MAX_PEAK);

		immut float max_luma = luminance(max_color);
		max_color = mix(vec3(max_luma), max_color, 0.1 * DCG_COLOR) - min_val;

		// I know this `frameTime` usage is probably technically wrong but it works
		// edit: i should fix it https://discord.com/channels/237199950235041794/736928196162879510/1287535068029522040
		dcg.max_color = mix(dcg.max_color, max_color, min(1.0, DCG_COLOR_SPEED * frameTime));
		dcg.min_val = mix(dcg.min_val, min_val, min(1.0, DCG_BP_SPEED * frameTime));
		dcg.max_luma = mix(dcg.max_luma, max_luma, min(1.0, DCG_EXP_SPEED * frameTime));
	#endif
}