#include "/lib/core.glsl"
#include "/lib/config.glsl"

#if COLORED_LIGHT || GLOSSY_REFLECTIONS
	layout(local_size_x = 1024, local_size_y = 1, local_size_z = 1) in;

	uniform layout(rg32ui) restrict uimage1D lightIndex;
	layout(shared, binding = 1) restrict buffer lightIndexLen { int light_index; };

	shared uint sh_index_size;
	shared int sh_culled_size;
	shared uvec2[INDEX_SIZE] sh_index;
#else
	layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
#endif

const ivec3 workGroups = ivec3(1, 1, 1);

void main() {
	#if COLORED_LIGHT || GLOSSY_REFLECTIONS
		if (gl_LocalInvocationIndex == 0u) {
			sh_index_size = light_index;
			sh_culled_size = 0;
		}

		barrier();

		immut uint index_size = sh_index_size;
		for (uint i = gl_LocalInvocationIndex; i < index_size; i += gl_WorkGroupSize.x) sh_index[i] = imageLoad(lightIndex, int(i)).rg;

		barrier();

		for (uint i = gl_LocalInvocationIndex; i < index_size; i += gl_WorkGroupSize.x) {
			immut uvec2 light = sh_index[i];

			bool unique = true;
			for (uint j = 0u; j < i; ++j) if (sh_index[j].x == light.x) unique = false;

			// what happens if different faces are different colors? race condition?
			// maybe we should do some kind of mixing here
			if (unique) imageStore(lightIndex, atomicAdd(sh_culled_size, 1), uvec4(light, 0u, 0u));
		}

		barrier();

		if (gl_LocalInvocationIndex == 0u) light_index = sh_culled_size;
	#endif
}