#include "/lib/core.glsl"
#include "/lib/config.glsl"

/* RENDERTARGETS: 2 */
layout(location = 0) out uvec2 outColor;

uniform float alphaTestRef;
uniform sampler2D gtexture;

in VertexData {
	vec2 pix;
	flat vec2 light;
	flat vec3 color;
	flat vec3 normal;
} v;

#include "/lib/fast_math.glsl"
#include "/lib/rand.glsl"
#include "/lib/dithered_unorm.glsl"
#include "/lib/octa_normal.glsl"
#include "/lib/specular.glsl"

void main() {
	vec4 color = texelFetch(gtexture, ivec2(v.pix), 0);
	if (color.a < alphaTestRef) discard;

	immut float smoothness =
		#if SM && defined MC_SPECULAR_MAP
			smoothness(texelFetch(specular, ivec2(v.pix), 0).r);
		#else
			smoothness(luminance(color.rgb));
		#endif

	immut uvec2 pix = uvec2(gl_FragCoord.xy);

	outColor = uvec2(
		packUnorm4x8(vec4(color.rgb * v.color, smoothness)),
		packUnorm4x8(vec4(octa_encode(v.normal), dithered_gamma2(v.light.xyy, pix, rand(pix), UN8).xy))
	);
}