#include "/lib/core.glsl"

uniform mat3 normalMatrix;
uniform mat4 gbufferModelViewInverse, modelViewMatrix, projectionMatrix, textureMatrix;
uniform sampler2D gtexture;

in ivec2 vaUV2;
in vec2 vaUV0;
in vec3 vaNormal, vaPosition;
in vec4 vaColor;

out VertexData {
	vec2 pix;
	flat vec2 light;
	flat vec3 color;
	flat vec3 normal;
} f;

void main() {
	immut vec4 view = modelViewMatrix * vec4(vaPosition, 1.0);
	vec4 clip = projectionMatrix * view;
	clip.z /= MC_HAND_DEPTH; // :/
	gl_Position = clip;

	f.pix = (textureMatrix * vec4(vaUV0, 0.0, 1.0)).xy * textureSize(gtexture, 0);
	f.color = vaColor.rgb;
	f.normal = mat3(gbufferModelViewInverse) * normalize(normalMatrix * vaNormal);
	f.light = ((
		mat4(0.00390625, 0.0, 0.0, 0.0, 0.0, 0.00390625, 0.0, 0.0, 0.0, 0.0, 0.00390625, 0.0, 0.03125, 0.03125, 0.03125, 1.0) *
		vec4(vaUV2, 0.0, 1.0)
	).xy - 1.0/32.0) * 16.0/15.0;
}