#include "/lib/core.glsl"
#include "/lib/config.glsl"

/* RENDERTARGETS: 1 */
layout(location = 0) out vec4 outColor;

uniform float alphaTestRef, moonSize;
uniform vec3 shadowLightPosition;
uniform sampler2D gtexture;

#include "/lib/srgb.glsl"

#ifndef NO_SKYLIGHT
	uniform float rainStrength;

	#include "/lib/skylight.glsl"
	#include "/lib/brdf.glsl"
	#include "/lib/specular.glsl"
#endif

in VertexData {
	vec2 pix;
	vec3 view;
	vec3 color;
	flat vec3 normal;
} v;

void main() {
	vec4 color = texelFetch(gtexture, ivec2(v.pix), 0);
	if (color.a < alphaTestRef) discard;

	color.rgb = linear(color.rgb * v.color);

	immut vec3 sun = shadowLightPosition * 0.01;
	float lambertian = dot(v.normal, sun);

	#ifdef NO_SKYLIGHT
		color.rgb *= mix(max(lambertian, 0.5), 1.0, 0.5);
	#else
		vec3 lighting = vec3(AMBIENT);

		if (lambertian > 0.01) {
			immut float smoothness =
				#if SM && defined MC_SPECULAR_MAP
					smoothness(texelFetch(specular, ivec2(v.pix), 0).r);
				#else
					smoothness(luminance(color.rgb));
				#endif

			lambertian += brdf(lambertian, normalize(v.normal), normalize(v.view), sun, 1.0 - smoothness) * float(SPECULAR * 0.1);

			lighting = fma(skylight(), lambertian.xxx, lighting);
		}

		color.rgb *= lighting;
	#endif

	outColor = color;
}