#include "/lib/core.glsl"
#include "/lib/config.glsl"

uniform mat3 normalMatrix;
uniform mat4 modelViewMatrix, projectionMatrix, textureMatrix;
uniform sampler2D gtexture;

in vec2 vaUV0;
in vec3 vaNormal, vaPosition;
in vec4 vaColor;

layout(shared, binding = 1) restrict readonly buffer lightIndexLen { int light_index; };

#include "/lib/fast_math.glsl"
#include "/lib/vertex_sample_index.glsl"

out VertexData {
	vec2 pix;
	vec3 view;
	vec3 color;
	flat vec3 normal;
} f;

void main() {
	immut vec4 view = modelViewMatrix * vec4(vaPosition, 1.0);
	gl_Position = projectionMatrix * view;

	immut vec3 normal = normalMatrix * vaNormal;

	f.pix = (textureMatrix * vec4(vaUV0, 0.0, 1.0)).xy * textureSize(gtexture, 0);
	f.color = vaColor.rgb * get_fancy_light(view.xyz, normal);
	f.normal = normal;
	f.view = view.xyz;
}