uniform layout(rgb10_a2) restrict writeonly image2D tempCol;

uniform vec2 pixSize;

#include "/lib/luma.glsl"

void main() {
	immut ivec2 pix = ivec2(gl_GlobalInvocationID.xy);
	vec3 color = texelFetch(colortex0, pix, 0).rgb;

	/*
		https://github.com/kosua20/Rendu/blob/master/resources/common/shaders/screens/fxaa.frag

		MIT License

		Copyright (c) 2017 Simon Rodriguez

		Permission is hereby granted, free of charge, to any person obtaining a copy
		of this software and associated documentation files (the "Software"), to deal
		in the Software without restriction, including without limitation the rights
		to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
		copies of the Software, and to permit persons to whom the Software is
		furnished to do so, subject to the following conditions:

		The above copyright notice and this permission notice shall be included in all
		copies or substantial portions of the Software.

		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
		IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
		FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
		AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
		LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
		OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
		SOFTWARE.
	*/

	immut float luma_down = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(0, -1)).rgb));
	immut float luma_up = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(0, 1)).rgb));
	immut float luma_left = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(-1, 0)).rgb));
	immut float luma_right = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(1, 0)).rgb));

	immut float luma_center = luminance(linearize_gamma2(color));

	immut float luma_min = min(luma_center, min(min(luma_down, luma_up), min(luma_left, luma_right)));
	immut float luma_max = max(luma_center, max(max(luma_down, luma_up), max(luma_left, luma_right)));

	immut float luma_range = luma_max - luma_min;

	if (luma_range > max(0.0312, luma_max * 0.125)) {
		immut float luma_down_left = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(-1, -1)).rgb));
		immut float luma_up_right = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(1, 1)).rgb));
		immut float luma_up_left = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(-1, 1)).rgb));
		immut float luma_down_right = luminance(linearize_gamma2(texelFetchOffset(colortex0, pix, 0, ivec2(1, -1)).rgb));

		immut float luma_down_up = luma_down + luma_up;
		immut float luma_left_right = luma_left + luma_right;

		immut float luma_left_corners = luma_down_left + luma_up_left;
		immut float luma_down_corners = luma_down_left + luma_down_right;
		immut float luma_right_corners = luma_down_right + luma_up_right;
		immut float luma_up_corners = luma_up_right + luma_up_left;

		immut float edge_horizontal =
			abs(-2.0 * luma_left + luma_left_corners) +
			abs(-2.0 * luma_center + luma_down_up) * 2.0 +
			abs(-2.0 * luma_right + luma_right_corners);
		immut float edge_vertical =
			abs(-2.0 * luma_up + luma_up_corners) +
			abs(-2.0 * luma_center + luma_left_right) * 2.0 +
			abs(-2.0 * luma_down + luma_down_corners);

		immut bool is_horizontal = (edge_horizontal >= edge_vertical);

		immut float luma_1 = is_horizontal ? luma_down : luma_left;
		immut float luma_2 = is_horizontal ? luma_up : luma_right;
		immut float gradient_1 = abs(luma_1 - luma_center);
		immut float gradient_2 = abs(luma_2 - luma_center);

		immut float gradient_scaled = 0.25 * max(gradient_1, gradient_2);

		float step_length = is_horizontal ? pixSize.y : pixSize.x;

		float luma_local_average = 0.0;

		if (gradient_1 >= gradient_2) {
			step_length = -step_length;
			luma_local_average = 0.5 * (luma_1 + luma_center);
		} else {
			luma_local_average = 0.5 * (luma_2 + luma_center);
		}

		immut vec2 coord = (gl_GlobalInvocationID.xy + 0.5) * pixSize;

		vec2 current_uv = coord;

		if (is_horizontal) {
			current_uv.y += step_length * 0.5;
		} else {
			current_uv.x += step_length * 0.5;
		}

		immut vec2 offset = is_horizontal ? vec2(pixSize.x, 0.0) : vec2(0.0, pixSize.y);
		vec2 uv1 = current_uv - offset;
		vec2 uv2 = current_uv + offset;

		float luma_end_1 = luminance(linearize_gamma2(textureLod(colortex0, uv1, 0.0).rgb)) - luma_local_average;
		float luma_end_2 = luminance(linearize_gamma2(textureLod(colortex0, uv2, 0.0).rgb)) - luma_local_average;

		bool reached_1 = abs(luma_end_1) >= gradient_scaled;
		bool reached_2 = abs(luma_end_2) >= gradient_scaled;

		bool reached_both = reached_1 && reached_2;

		if (!reached_1) uv1 -= offset;
		if (!reached_2) uv2 += offset;

		if (!reached_both) {
			const float quality[] = float[](1.0, 1.0, 1.0, 1.5, 2.0, 2.0, 2.0, 2.0, 4.0, 8.0);

			for(uint i = 0u; i < quality.length(); ++i) {
				if (!reached_1) luma_end_1 = luminance(linearize_gamma2(textureLod(colortex0, uv1, 0.0).rgb)) - luma_local_average;
				if (!reached_2) luma_end_2 = luminance(linearize_gamma2(textureLod(colortex0, uv2, 0.0).rgb)) - luma_local_average;

				reached_1 = abs(luma_end_1) >= gradient_scaled;
				reached_2 = abs(luma_end_2) >= gradient_scaled;
				reached_both = reached_1 && reached_2;

				if (!reached_1) uv1 = fma(offset, (-quality[i]).xx, uv1);
				if (!reached_2) uv2 = fma(offset, quality[i].xx, uv2);

				if (reached_both) break;
			}
		}

		immut float distance_1 = is_horizontal ? coord.x - uv1.x : coord.y - uv1.y;
		immut float distance_2 = is_horizontal ? uv2.x - coord.x : uv2.y - coord.y;

		immut bool correct_variation = ((distance_1 < distance_2 ? luma_end_1 : luma_end_2) < 0.0) != luma_center < luma_local_average;

		float final_offset = correct_variation ? -min(distance_1, distance_2) / (distance_1 + distance_2) + 0.5 : 0.0;

		immut float luma_average = (2.0 * (luma_down_up + luma_left_right) + luma_left_corners + luma_right_corners) / 12.0;
		immut float sub_pixel_offset_1 = min(abs(luma_average - luma_center) / luma_range, 1.0);
		immut float sub_pixel_offset_2 = fma(sub_pixel_offset_1, -2.0, 3.0) * sub_pixel_offset_1 * sub_pixel_offset_1;

		final_offset = max(final_offset, sub_pixel_offset_2 * sub_pixel_offset_2 * 0.75);

		vec2 final_uv = coord;
		final_uv[uint(is_horizontal)] += final_offset * step_length;

		color = textureLod(colortex0, final_uv, 0.0).rgb;
	}

	imageStore(tempCol, pix, vec4(color, 0.0));
}