/*
	https://github.com/GPUOpen-LibrariesAndSDKs/FidelityFX-SDK/blob/main/sdk/include/FidelityFX/gpu/fsr1/ffx_fsr1.h#L1093

	Copyright (C) 2024 Advanced Micro Devices, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files(the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and /or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions :

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	THE SOFTWARE.
*/

#define UN8 255u
#define UN10 1023u

vec3 dithered_gamma2(vec3 color, uvec2 pix, float noise, uint uint_max) {
	immut float rcp_max = 1.0 / uint_max;

	immut vec3 n = floor(sqrt(color) * uint_max) * rcp_max;
	vec3 b = n + rcp_max;
	b *= b;

	return fma(step_zero(fract(fma(pix.x + noise, fma(sqrt(5.0), 0.5, 0.5), pix.y / 3.69)) - (color - b) / (fma(n, n, -b))), vec3(rcp_max), n);
}

vec3 linearize_gamma2(vec3 color) {
	return color * color;
}