#include "/lib/core.glsl"
#include "/lib/config.glsl"

/* RENDERTARGETS: 2 */
layout(location = 0) out uvec2 outColor;

uniform sampler2D gtexture;

#ifdef NO_NORMAL
	uniform mat3 normalMatrix;
	uniform mat4 gbufferModelViewInverse;
#endif

#ifdef ALPHA_CHECK
	uniform float alphaTestRef;

	//layout(depth_greater) out float gl_FragDepth; // is this right?
#else
	layout(depth_unchanged) out float gl_FragDepth;
#endif

in VertexData {
	vec2 coord, light;
	vec3 color;

	#ifndef NO_NORMAL
		flat vec3 normal;
	#endif
} v;

#include "/lib/fast_math.glsl"
#include "/lib/rand.glsl"
#include "/lib/dithered_unorm.glsl"
#include "/lib/octa_normal.glsl"
#include "/lib/specular.glsl"

void main() {
	#ifdef TEX_ALPHA
		vec4 color = texture(gtexture, v.coord);
	#else
		vec3 color = texture(gtexture, v.coord).rgb;
	#endif

	#ifdef ALPHA_CHECK
		if (color.a < alphaTestRef) discard;
	#endif

	color.rgb *= v.color;

	// todo!() normal map here
	immut vec3 normal =
		#ifdef NO_NORMAL
			mat3(gbufferModelViewInverse) * normalMatrix * vec3(0.0, -1.0, 0.0);
		#else
			v.normal;
		#endif

	immut float smoothness =
		#if SM && defined MC_SPECULAR_MAP
			smoothness(texture(specular, v.coord).SM_CH);
		#else
			smoothness(luminance(color.rgb));
		#endif

	// See layout in `deferred.csh`
	immut uvec2 pix = uvec2(gl_FragCoord.xy);

	outColor = uvec2(
		packUnorm4x8(vec4(color.rgb, smoothness)),
		packUnorm4x8(vec4(
			octa_encode(normal),
			dithered_gamma2(v.light.xyy, pix, rand(pix), UN8).xy
		))
	);
}