#include "/lib/core.glsl"
#include "/lib/config.glsl"

#ifdef EMISSIVE_REDSTONE_BLOCK
#endif
#ifdef EMISSIVE_EMERALD_BLOCK
#endif
#ifdef EMISSIVE_LAPIS_BLOCK
#endif

uniform float far;
uniform vec3 cameraPositionFract;
uniform mat4 gbufferModelViewInverse, modelViewMatrix, projectionMatrix, textureMatrix;

#ifndef NO_NORMAL
	uniform mat3 normalMatrix;
#endif

#ifdef TERRAIN
	uniform vec3 chunkOffset;

	in vec2 mc_Entity;
	in vec4 at_midBlock;

	#if WAVES && defined SOLID_TERRAIN
		#include "/lib/wave.glsl"
	#endif

	#if COLORED_LIGHT || GLOSSY_REFLECTIONS
		in vec2 mc_midTexCoord;

		uniform sampler2D gtexture;
		layout(shared, binding = 1) restrict buffer lightIndexLen { int light_index; };
		uniform layout(rg32ui) restrict writeonly uimage1D lightIndex;
	#endif
#endif

#ifdef ENTITY_COLOR
	uniform vec4 entityColor;
#endif

#ifndef NO_NORMAL
	in vec3 vaNormal;
#endif

in ivec2 vaUV2;
in vec2 vaUV0;
in vec3 vaPosition;
in vec4 vaColor;

out VertexData {
	vec2 coord, light;
	vec3 color;

	#ifndef NO_NORMAL
		flat vec3 normal;
	#endif
} f;

#include "/lib/srgb.glsl"

void main() {
	vec3 model = vaPosition;

	#ifdef TERRAIN
		model += chunkOffset;

		#ifdef SOLID_TERRAIN
			if (mc_Entity.y > 0.5) {
				#if WAVES
					model.y += wave(model.xz);
				#endif
			}
		#endif
	#endif

	immut vec4 view = modelViewMatrix * vec4(model, 1.0);
	immut float view_len = length(view.xyz);

	//if (view_len < far) {
		immut vec2 coord = (textureMatrix * vec4(vaUV0, 0.0, 1.0)).xy;
		vec3 color = vaColor.rgb;

		#ifdef ENTITY_COLOR
			color.rgb = mix(color.rgb, entityColor.rgb, entityColor.a);
		#endif

		vec2 light = ((
			mat4(0.00390625, 0.0, 0.0, 0.0, 0.0, 0.00390625, 0.0, 0.0, 0.0, 0.0, 0.00390625, 0.0, 0.03125, 0.03125, 0.03125, 1.0) *
			vec4(vaUV2, 0.0, 1.0)
		).xy - 1.0/32.0) * 16.0/15.0; // lm_coord scaled to 0-1

		#ifdef TERRAIN
			immut float emission = min((max(mc_Entity.x, 0.0) + at_midBlock.w) / 15.0, 1.0);
			light.x = min(fma(emission, 0.3, max(light.x, emission)), 1.0);

			#include "/lib/index_light.glsl"
		#endif

		f.coord = (textureMatrix * vec4(coord, 0.0, 1.0)).xy;
		f.color = color;
		f.light = light;

		#ifndef NO_NORMAL
			f.normal = mat3(gbufferModelViewInverse) * normalize(normalMatrix * vaNormal);
		#endif

		gl_Position = projectionMatrix * view;
	//} else gl_Position = vec4(0.0, 0.0, 8192.0, 1.0);
}