#include "/lib/core.glsl"
#include "/lib/config.glsl"

/* RENDERTARGETS: 1 */
layout(location = 0) out vec4 outColor;

uniform float far, frameTimeCounter;
uniform mat4 gbufferModelViewInverse;
uniform sampler2D gtexture;

#ifdef ALPHA_CHECK
	uniform float alphaTestRef;
#endif

in VertexData {
	vec2 coord;
	vec3 s_ndc, view;

	#ifndef NO_NORMAL
		vec3 normal;
	#endif

	#ifdef TERRAIN
		vec4 color;
	#else
		vec3 color;
	#endif
} v;

#include "/lib/fast_math.glsl"

#ifndef NETHER
	uniform float rainStrength, moonSize;
	uniform vec3 shadowLightPosition;
	uniform sampler2D shadowcolor0;
	uniform sampler2DShadow shadowtex1;

	#include "/lib/skylight.glsl"
	#include "/lib/sample_shadow.glsl"
#endif

#include "/lib/srgb.glsl"
#include "/lib/rand.glsl"
#include "/lib/fog.glsl"
#include "/lib/specular.glsl"

void main() {
	vec4 color = texture(gtexture, v.coord);

	#ifdef ALPHA_CHECK
		if (color.a < alphaTestRef) discard;
	#endif

	immut float smoothness =
		#if SM && defined MC_SPECULAR_MAP
			smoothness(texture(specular, v.coord).r);
		#else
			smoothness(luminance(color.rgb));
		#endif

	#ifdef TERRAIN
		color *= v.color;
	#else
		color.rgb *= v.color;
	#endif

	color.rgb = linear(color.rgb);

	immut vec3 n_view = normalize(v.view);
	immut float fog = fog(length(v.view));

	#ifndef NETHER
		immut vec3 sun = shadowLightPosition * 0.01;

		#ifndef NO_NORMAL
			float lambertian = dot(v.normal, sun);
		#endif

		vec3 lighting = AMBIENT * vec3(0.8, 0.9, 1.0);

		if (
			fog < 0.999
			#if !SSS && !defined NO_NORMAL
				&& lambertian > 0.01
			#endif
		) lighting += sample_shadow(
			v.view,
			n_view,
			v.s_ndc,
			sun,
			skylight()

			#ifndef NO_NORMAL
				, // :concern:
				v.normal,
				lambertian,
				smoothness
			#endif
		);

		color.rgb *= lighting;
	#endif

	#if defined END || defined NETHER
		color.rgb = mix(color.rgb, linear(fogColor), fog);
	#else
		immut float sky_fog = sky_fog(max((mat3(gbufferModelViewInverse) * n_view).y, 0.0));

		color.rgb = mix(color.rgb, sky(sky_fog, n_view), fog);
	#endif

	outColor = color;
}