#if TONEMAP == 10 // SBDT
	float sbdt_curve(float v) {
		immut float c = v + v*v + 0.5 * v*v*v;
		return c / (1.0 + c);
	}

	vec3 sbdt_curve(vec3 color) {
		immut vec3 c = color + color*color + 0.5 * color*color*color;
		return c / (1.0 + c);
	}
#endif

vec3 tonemap(vec3 color) {
	#if TONEMAP == 0 // Clamp
		return clamp(color, 0.0, 1.0);
	#elif TONEMAP == 1 // Uchimura
		// Based on: https://github.com/dmnsgn/glsl-tone-map/blob/main/uchimura.glsl

		immut vec3 w0 = 1.0 - smoothstep(0.0, 0.22, color);
		immut vec3 w2 = step(0.532, color);

		return 0.22 * pow(color / 0.22, vec3(1.33)) * w0 + color * (1.0 - w0 - w2) + (1.0 - 0.468 * exp((0.532 - color) / 0.468)) * w2;
	#elif TONEMAP == 2 // Reinhard-Jodie
		color = pow(color, uvec3(5u));

		immut vec3 tv = color / (1.0 + color);

		return pow(mix(color / (dot(color, vec3(0.2125, 0.7154, 0.0721)) + 1.0), tv, tv), vec3(0.2));
	#elif TONEMAP == 3 // ACES
		// Based on: https://github.com/dmnsgn/glsl-tone-map/blob/main/aces.glsl

		return color * (2.51 * color + 0.03) / (color * (2.43 * color + 0.59) + 0.14);
	#elif TONEMAP == 4 // Lottes
		// Based on: https://github.com/dmnsgn/glsl-tone-map/blob/main/lottes.glsl

		const float a = (pow(8.0, 1.5632) - pow(0.18, 1.5632)) * 0.267;
		const float b = (pow(8.0, 1.6) * 0.267 - pow(0.18, 1.6)) / a;
		const float c = (pow(8.0, 1.5632) * pow(0.18, 1.6) - pow(8.0, 1.6) * pow(0.18, 1.5632) * 0.267) / a;

		return pow(color, vec3(1.6)) / (pow(color, vec3(1.5632)) * b + c);
	#elif TONEMAP == 5 // Unreal
		// Based on: https://github.com/dmnsgn/glsl-tone-map/blob/main/unreal.glsl

		return pow(color / (color + 0.155) * 1.019, vec3(2.2));
	#elif TONEMAP == 6 // Filmic
		// Based on: https://github.com/dmnsgn/glsl-tone-map/blob/main/filmic.glsl

		color = max(color - 0.004, 0.0);

		return pow(color * (6.2 * color + 0.5) / (color * (6.2 * color + 1.7) + 0.06), vec3(2.2));
	#elif TONEMAP == 7 // Burgess HDR
		color = max(mix(color * 0.6, color * 1.2, color) - 0.004, 0.0);

		return color * (6.2 * color + 0.05) / (color * (6.2 * color + 2.3) + 0.06);
	#elif TONEMAP == 8 // Uncharted 2
		// Based on: https://github.com/dmnsgn/glsl-tone-map/blob/main/uncharted2.glsl

		color *= 2.0;

		return ((color * (0.15 * color + 0.05) + 0.004) / (color * (0.15 * color + 0.5) + 0.06) - 0.2/3.0) / (19.38/24.476 - 0.2/3.0);
	#elif TONEMAP == 9 // PBR Neutral
		// Based on: https://github.com/KhronosGroup/ToneMapping/blob/main/PBR_Neutral/pbrNeutral.glsl

		const float startCompression = 0.76;
		const float desaturation = 0.15;

		immut float x = min(color.r, min(color.g, color.b));
		color -= x < 0.08 ? x - 6.25 * x*x : 0.04;

		immut float peak = max(color.r, max(color.g, color.b));
		if (peak < startCompression) return color;

		const float d = 1.0 - startCompression;
		immut float newPeak = 1.0 - d*d / (peak + d - startCompression);
		color *= newPeak / peak;

		return mix(color, vec3(newPeak), 1.0 - 1.0 / (desaturation * (peak - newPeak) + 1.0));
	#elif TONEMAP == 10 // SBDT
		// Based on: https://github.com/bevyengine/bevy/blob/main/crates/bevy_core_pipeline/src/tonemapping/tonemapping_(packed.wgsl

		immut vec3 ycbcr = color * mat3(
			0.2126, 0.7152, 0.0722,
			-0.1146, -0.3854, 0.5,
			0.5, -0.4542, -0.0458
		);

		immut float bt = sbdt_curve(length(ycbcr.gb) * 2.4);
		immut float desat = max(bt * 0.8 - 0.56, 0.0);

		return mix(
			color * max(0.0, sbdt_curve(ycbcr.r) / max(1e-5, dot(color, vec3(0.2126, 0.7152, 0.0722)))),
			sbdt_curve(mix(color, ycbcr.rrr, desat * desat)),
			bt * bt
		) * 0.97;
	#endif
}