#include "/lib/core.glsl"
#include "/lib/config.glsl"
#include "/lib/tile.glsl"

uniform sampler2D depthtex0;
uniform layout(HDR_FMT) restrict writeonly image2D colorimg1;

uniform float far;
uniform vec2 pixSize;
uniform mat4 gbufferProjectionInverse;
uniform usampler2D colortex2;

#define NETHER
#include "/lib/fast_math.glsl"
#include "/lib/srgb.glsl"
#include "/lib/fog.glsl"

void main() {
	immut ivec2 texel = ivec2(gl_GlobalInvocationID.xy);
	immut float depth = texelFetch(depthtex0, texel, 0).r;
	immut vec2 coord = fma(gl_GlobalInvocationID.xy, pixSize, 0.5 * pixSize);
	immut vec4 view_undiv = gbufferProjectionInverse * vec4(fma(vec3(coord, depth), vec3(2.0), vec3(-1.0)), 1.0);
	immut vec3 view = view_undiv.xyz / view_undiv.w;

	vec3 color;

	immut vec3 fog_color = linear(fogColor);
	
	if (depth >= 1.0) {
		color = fog_color;
	} else {
		immut uvec2 gbuffer_data = texelFetch(colortex2, texel, 0).rg;
		/* |-- R --|-- G --]
		   └8 8 8 8┴8 8 8 8┘
			| | | | | | | |
			| | | | | | └[light]: 2x8
			| | | | └[normal]: 2x8
			| | | └[smoothness]: 1x8
			└[color]: 3x8
		*/

		immut float fog = fog(length(view));
		color = mix(linear(unpackUnorm4x8(gbuffer_data.x).rgb), fog_color, fog);
	}

	imageStore(colorimg1, texel, vec4(
		color,
		0.0
	));
}